#include "SunRise.h"

#include <windows.h>
#include <math.h>

const double PI = 3.14159265358979323846;

SunRise::SunRise(const double & lat, const double & longi,const bool &sysTime){
	this->sysTime=sysTime;
	this->lat=lat;
	this->longi=longi;
	this->jourJulien=0;
	this->jourJulienCorrige=0;
	this->ascensionD=this->declin=angleHoraire=0;
	this->altitude=this->azimut=0;
	this->heureSiderale=0;
	this->elapsedTime=0;

	//Si on utilise l'heure systeme on initialise a l'heure systeme
	if(sysTime){
		SYSTEMTIME utcTime;
		GetSystemTime(&utcTime);
		this->jour=utcTime.wDay;
		this->mois=utcTime.wMonth;
		this->annee=utcTime.wYear;
		this->heure=utcTime.wHour;
		this->minute=utcTime.wMinute;
		this->seconde=utcTime.wSecond;
	}else{//Sinon on initialise au 01/01/2006 a 12 heure
		this->jour=25;
		this->mois=10;
		this->annee=2006;
		this->heure=18;
		this->minute=00;
		this->seconde=0;
	}
}

void SunRise::updateJourJulien(){
		
	double month, year, day, a, b;
  
    day=this->jour+this->heure/24.0+this->minute/1440.0+this->seconde/86400.0;
    year=this->annee;
    month=this->mois;
  
    if(month < 3)
    {
         year=year-1;
         month=month+12;
    }
  
    a=(int)(year/100.0);
    b=2-a+(int)(a/4.0);
 
	this->jourJulien=(int)(365.25*(year+4716.0))+(int)(30.6001*(month+1.0))+day+b-1524.5;
	this->jourJulienCorrige=this->jourJulien-2451545.0;
}


void SunRise::updateAscen_Declin(){
	
	double g,q,l,e;
	
	g=357.529+0.98560028*this->jourJulienCorrige;
	q=280.459+0.98564736*this->jourJulienCorrige;
	l=q+1.915*sin(g*PI/180.0)+0.020*sin(2*g*PI/180.0);
	e=23.439-0.00000036*this->jourJulienCorrige;
	 
	this->ascensionD=atan(cos(e*PI/180.0)*sin(l*PI/180.0)/cos(l*PI/180.0))*(180.0/PI)/15.0;
	if(cos(l*PI/180.0)<0)
	{
			this->ascensionD=12.0+this->ascensionD;
	}
	if(cos(l*PI/180.0)>0 && sin(l*PI/180.0)<0)
	{
			this->ascensionD=this->ascensionD+24.0;
	}

	this->declin=asin(sin(e*PI/180.0)*sin(l*PI/180.0))*180.0/PI;

}

void SunRise::updateAngleHoraire(){
	
	double nb_siecle=this->jourJulienCorrige/36525.0;
	double heure_siderale1=(24110.54841+(8640184.812866*nb_siecle)+(0.093104*(nb_siecle*nb_siecle))-(0.0000062*(nb_siecle*nb_siecle*nb_siecle)))/3600.0;
	double heure_siderale2=((heure_siderale1/24.0)-int(heure_siderale1/24.0))*24.0;
	 
	double angleH=360.0*heure_siderale2/23.9344;
	double angleT=(this->heure-12.0+this->minute/60.0+this->seconde/3600.0)*360.0/23.9344;
	double angle=angleT+angleH;
	 
	this->angleHoraire=angle-this->ascensionD*15.0+this->longi;
}

void SunRise::updateAltAzi(){
	this->altitude=asin(sin(this->declin*PI/180.0)*sin(this->lat*PI/180.0)-cos(this->declin*PI/180.0)*cos(this->lat*PI/180.0)*cos(this->angleHoraire*PI/180.0))*180.0/PI;

	this->azimut=acos((sin(this->declin*PI/180.0)-sin(this->lat*PI/180.0)*sin(this->altitude*PI/180.0))/(cos(this->lat*PI/180.0)*cos(this->altitude*PI/180.0)))*180.0/PI;
	double sinazimut=(cos(this->declin*PI/180.0)*sin(this->angleHoraire*PI/180.0))/cos(this->altitude*PI/180.0);
	if(sinazimut<0)
	{
	this->azimut=360-this->azimut;
	}
}

void SunRise::update(double dt){

	if(sysTime){
		SYSTEMTIME utcTime;
		GetSystemTime(&utcTime);
		this->jour=utcTime.wDay;
		this->mois=utcTime.wMonth;
		this->annee=utcTime.wYear;
		this->heure=utcTime.wHour;
		this->minute=utcTime.wMinute;
		this->seconde=utcTime.wSecond;
	}else{
		this->updateTime(dt);
	}

	updateJourJulien();
	updateAscen_Declin();
	updateAngleHoraire();
	updateAltAzi();
}


void SunRise::set_Date(unsigned int jour,
					unsigned int mois,
					unsigned int annee,
					unsigned int heure, 
					unsigned int minute, 
					unsigned int seconde){
	this->jour=jour;
	this->mois=mois;
	this->annee=annee;
	this->heure=heure;
	this->minute=minute;
	this->seconde=seconde;
}


bool isLeapYear(unsigned int &year) {
   return ((year % 4 == 0) && (year % 100 != 0))
 	|| (year % 400 == 0);
}

void SunRise::updateTime(double dt){
	
	if(dt==0) return;
	
	this->elapsedTime+=dt;

	
	seconde+=dt;

	if (seconde>=60){
		double tempseconde=0+ modf(seconde,&seconde);
		minute=minute + (int)(seconde/60);
		seconde=tempseconde;
	}else return;

	if (minute>=60){
		heure=heure + (int)(minute/60);
		minute=minute%60;
	}else return;

	if (heure >= 24){
		jour=jour+ (int)(heure/24) ;
		heure=heure%24;
	}else return;

	//Si mois a 31 jours
	if (mois==1 || mois ==3 || mois == 5|| mois ==7 || mois==8 || mois==10 || mois == 12){
		if(jour>31){
			jour=1;
			++mois;
			if (mois >12){
				mois=0;
				++annee;
			}
		}
	}else if (mois!=2){//Sinon si mois a 30 jours
		if(jour>30){
			jour=1;
			++mois;
			if (mois >12){
				mois=0;
				++annee;
			}
		}
	}else{ //sinon si fevrier
		if(isLeapYear(annee)){
			if(jour>29){
				jour=1;
				++mois;
				if (mois >12){
					mois=0;
					++annee;
				}
		}
		}else{
			if(jour>28){
				jour=1;
				++mois;
				if (mois >12){
					mois=0;
					++annee;
				}
			}
		}
	}
}